/* ****************************************************************************
 * Copyright 2019 Open Systems Development BV                                 *
 *                                                                            *
 * Permission is hereby granted, free of charge, to any person obtaining a    *
 * copy of this software and associated documentation files (the "Software"), *
 * to deal in the Software without restriction, including without limitation  *
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,   *
 * and/or sell copies of the Software, and to permit persons to whom the      *
 * Software is furnished to do so, subject to the following conditions:       *
 *                                                                            *
 * The above copyright notice and this permission notice shall be included in *
 * all copies or substantial portions of the Software.                        *
 *                                                                            *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,   *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL    *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING    *
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER        *
 * DEALINGS IN THE SOFTWARE.                                                  *
 * ***************************************************************************/
#include "ormthread.h"
#include "ormhandler.h"
#include "log.h"

using namespace osdev::components;

ORMThread::ORMThread()
{
}

ORMThread::~ORMThread()
{
}

void ORMThread::run()
{
    OrmHandler *p_OrmHandler = new OrmHandler();

    // Connect incoming data
    connect( this, &ORMThread::signalSendData, p_OrmHandler, &OrmHandler::receiveData );
    // Cascade connect the rejectedData
    connect( p_OrmHandler, &OrmHandler::signalRejectedData, this, &ORMThread::signalRejectedData );

    p_OrmHandler->start();

    this->exec();
}

void ORMThread::dataToThread( const QSharedPointer<ORMRelData>& data )
{
    QCoreApplication::processEvents();
    LogDebug( "[ORMThread::dataToThread]", QString( "Data received for container : %1" )
              .arg( data->getMainTableName() ) );
    LogDebug( "[ORMThread::dataToThread]", data->asString() );

    emit signalSendData( data );
    QCoreApplication::processEvents();
}
